#!/usr/bin/env python

# The PyTone configuration file is split into three parts I -- III.
#
# In part I you most likely want to change the basedir option!
#
# In part IIa you can change the configuration of
# the player(s) you want to use.
# In part IIb you can change the list of active players.
#
# Part III, where the keybindings are defined, you probably want
# to leave alone for the moment.
#
# Part IV defines the used attribute/color scheme


##############################################################################
# I. general PyTone configuration
##############################################################################

# basedir: basedir for filesystem based navigation and auto
# registering of songs in database.
#
# !!! You almost certainly want to adjust this !!!
basedir = "/mnt/mp3"

# songdbbase: base name of songdb
#
# The song database stores all relevant informations of your songs.
# Here you can specify the prefix of the files it uses. The actual files
# are named "<songdbbase>_songs.db" and "<songdbbase>_artists.db"
songdbbase = "~/mp3"

# logfile: Name of the file, where the played songs are logged.
#
# Set this to None, if you don't want the logging.
logfile = "~/pytone.log"

# here you can configure, whether the filelist and the playlist window, 
# respectively, have scrollbars
filelistwinhasscrollbar = 1
playlistwinhasscrollbar = 0

# show input windows (for searching, file names, etc.) and mixer (if
# present), respectively, as a popup or, if not, in the status bar
inputwinpopup = 1
mixerwinpopup = 1

# mixer configuration (uncomment one):
# for possible mixer names, see the Python OSS module, which is
# necessary, if you want to use a mixer in PyTone

# mixer functionality disabled
# mixer = None

# use OSS mixer for master output level
# mixer = ("/dev/mixer", "SOUND_MIXER_VOLUME")

# use OSS mixer for PCM output level
mixer = ("/dev/mixer", "SOUND_MIXER_PCM")


##############################################################################
# IIa. player configuration
#
# general format: 
#   name:    name of player
#   options: dictionary with player options
##############################################################################

##############################################################################
# mad: internal player (using libmad)
#
# options:
#   device:              libao device ("oss", "alsa", "alsa09", "esd", etc.)
#   dsp:                 path of dsp used for output
#   bufsize:             0:  no buffering by PyTone
#                        >0: size of internal buffer used for output in kBytes
#                        Note: A large buffer may prevent clicking but
#                        leads to a delayed response for instance to play next
#                        song requests
#   autoplay:            automatic start of playing, if songs are in playlist 
#   crossfadingstart:    start crossfading, if currently played song has only
#                        mixingstart seconds left
#   crossfadingduration: duration of crossfading in seconds
##############################################################################

# define some example madplayers
# the first one uses /dev/dsp (with a 500 kByte large buffer) 
madplayer_oss_0 = {
    "name"   : "mad",
    "options": {"aodevice"           : "oss",
                "aooptions"          : {"dsp": "/dev/dsp"},
                "bufsize"            : 500,
                "autoplay"           : 1, 
                "crossfadingstart"   : 5,
                "crossfadingduration": 6}
    }


# the second one outputs to  /dev/dsp1 and is unbuffered
madplayer_oss_1 = {
    "name"   : "mad",
    "options": {"aodevice"           : "oss",
                "aooptions"          : {"dsp": "/dev/dsp1"},
                "bufsize"            : 0,
                "autoplay"           : 1, 
                "crossfadingstart"   : 5,
                "crossfadingduration": 6}
    }

# here comes a madplayer which outputs with a small buffer to aRts
madplayer_arts = {
    "name"   : "mad",
    "options": {"aodevice"           : "arts",
                "aooptions"          : {},
                "bufsize"            : 100,
                "autoplay"           : 1, 
                "crossfadingstart"   : 5,
                "crossfadingduration": 6}
    }

##############################################################################
# mpg123: external mpg321 or mpg123 based player
#
# options:
#   autoplay: automatic start of playing, if songs are in playlist
#   cmdline:  cmdline use for the start of the player
##############################################################################

# define two mpg123 type external players (here mpg321) using /dev/dsp
# and /dev/dsp1
mpg123player0 = {
    "name"   : "mpg123",
    "options": {"cmdline" : "/usr/bin/mpg321 --skip-printing-frames=5 -a /dev/dsp",
                "autoplay": 1}
    }

mpg123player1 = {
    "name"   : "mpg123",
    "options": {"cmdline": "/usr/bin/mpg321 --skip-printing-frames=5 -a /dev/dsp1",
                "autoplay": 1}
    }

##############################################################################
# xmms: external player using xmms
#
# options:
#   session:  xmms session number
#   autoplay: automatic start of playing, if songs are in playlist
#   noqueue:  0=no internal queue for song
#             1=internal queue for songs, such that crossfading (via xmms-crossfade)
#               is possible
##############################################################################

# xmms player without automatic start of playing and without queueing
# for crossfading
xmmsplayer = {
    "name"   : "xmms",
    "options": {"session" : 0,
                "autoplay": 1, 
                "noqueue" : 0}
    }

##############################################################################
# IIb. players: defines the list of active players
# 
# Note:
#  If you specify only one player, it will be used for playing of the songs in
#  the playlist.
#  If you specify two players, the first one is used for the playlist and
#  the second one always plays the currently selected song. For this latter
#  option you obviously need two outputs on your soundcard or two cards.
##############################################################################

# uncomment one of the following players definitions

# configurations with one player
players = [madplayer_oss_0]
# players = [mpg123player0]
# players = [xmmsplayer]

# a configuration with two players: the first one plays the songs in the
# playlist, the second one always plays the currently selected song
# use this, if you have to soundcards or one with two line outs

# players = [madplayer_oss_0, madplayer_oss_1]


##############################################################################
# III. key bindings
#
# dictionary with entries
#  "global":   global key bindings
#  "filelist": key bindings only active in filelist window
#  "playlist": key bindings only active in playlist window
# which map from
#   name of function to 
#   tuple of accepted keys (first one may be displayed in status bar)
##############################################################################

# we need the curses module for the key codes
import curses

# define convenience function: ctrl keys
def ctrl(char):
    """return ascii value for ctrl+<char>"""
    return ord(char.upper())-64


keybindings = {
    "global": {
          "refresh":           (ctrl("l"),),
          "exit":              (ctrl("x"),),     # has to be pressed two times in fast succ.
          "playerstart":       (ord("p"), ord("P")),
          "playernextsong":    (ord("n"), ord("N")),
          "playerstop":        (ord("S"),),
          "playlistdeleteplayedsongs":
                               (curses.KEY_BACKSPACE,),
          "playlistclear":     (ctrl("d"),),
          "playlistsave":      (ctrl("w"),),
          "playlistload":      (ctrl("r"),),
          "showhelp":          (ord("?"),),
          "volumeup":          (ord(")"),),
          "volumedown":        (ord("("),),
    },
    "filelist": {
          "selectnext":        (curses.KEY_DOWN,),
          "selectprev":        (curses.KEY_UP,),
          "selectnextpage":    (ctrl("n"), curses.KEY_NPAGE),
          "selectprevpage":    (ctrl("p"), curses.KEY_PPAGE),
          "selectfirst":       (ctrl("a"), curses.KEY_HOME),
          "selectlast":        (ctrl("e"), curses.KEY_END),
          "dirdown":           (curses.KEY_RIGHT, ord(" "), ord("\n"), curses.KEY_ENTER),
          "dirup":             (curses.KEY_LEFT,),
          "addsongtoplaylist": (ord(" "), ord("\n"), curses.KEY_ENTER, curses.KEY_RIGHT),
          "adddirtoplaylist":  (ord("i"), ord("I"), curses.KEY_IC, 1024+curses.KEY_RIGHT),
          "activateplaylist":  (ord("\t"),),
          "toggleview":        (ord("t"), ord("T")),
          "generaterandomlist":(ord("r"), ord("R")),
          "search":            (ord("/"), ctrl("s"))
          },
    "playlist": {
          "selectnext":        (curses.KEY_DOWN,),
          "selectprev":        (curses.KEY_UP,),
          "selectnextpage":    (ctrl("n"), curses.KEY_NPAGE),
          "selectprevpage":    (ctrl("p"), curses.KEY_PPAGE),
          "selectfirst":       (ctrl("a"), curses.KEY_HOME),
          "selectlast":        (ctrl("e"), curses.KEY_END),
          "moveitemup":        (ord("+"),),  
          "moveitemdown":      (ord("-"),),
          "deleteitem":        (ord("d"), ord("D"), curses.KEY_DC),
          "activatefilelist":  (ord("\t"), curses.KEY_LEFT, curses.KEY_RIGHT),
          }
    }


##############################################################################
# IV. attributes/colors
#
# dictionary "colors" with entries
#  "filelist": attributes/colors in filelist window
#  "playlist": attributes/colors in playlist window
# which map from type of item to color pair
##############################################################################

# first we initialize some colors. You may change this to change the
# look of PyTone.
curses.init_pair(1, curses.COLOR_WHITE,   curses.COLOR_BLACK)
curses.init_pair(2, curses.COLOR_BLUE,    curses.COLOR_BLACK)
curses.init_pair(3, curses.COLOR_WHITE,   curses.COLOR_RED)
curses.init_pair(4, curses.COLOR_BLUE,    curses.COLOR_RED)
curses.init_pair(5, curses.COLOR_GREEN,   curses.COLOR_BLACK)
curses.init_pair(6, curses.COLOR_CYAN,    curses.COLOR_BLACK)
curses.init_pair(7, curses.COLOR_MAGENTA, curses.COLOR_BLACK)
curses.init_pair(8, curses.COLOR_GREEN,   curses.COLOR_GREEN)
curses.init_pair(9, curses.COLOR_CYAN,   curses.COLOR_CYAN)
curses.init_pair(10, curses.COLOR_RED,   curses.COLOR_RED)

# example attribute/color scheme for color terminals
colors_colors = {
    "filelist": {
          "background":           curses.color_pair(1),
          "border":               curses.color_pair(5),
          "activeborder":         curses.color_pair(5) | curses.A_BOLD,
          "title":                curses.color_pair(5) | curses.A_BOLD,
          "scrollbar":            curses.color_pair(8),
          "scrollbarhigh":        curses.color_pair(9),
          "scrollbararrow":       curses.color_pair(6),
          "song":                 curses.color_pair(1),
          "directory":            curses.color_pair(2) | curses.A_BOLD,
          "selectedsong":         curses.color_pair(3),
          "selecteddirectory":    curses.color_pair(4) | curses.A_BOLD,
          },
    "playlist": {
          "background":           curses.color_pair(1),
          "border":               curses.color_pair(5),
          "activeborder":         curses.color_pair(5) | curses.A_BOLD,
          "title":                curses.color_pair(5) | curses.A_BOLD,
          "scrollbar":            curses.color_pair(8),
          "scrollbarhigh":        curses.color_pair(9),
          "scrollbararrow":       curses.color_pair(6),
          "unplayedsong":         curses.A_BOLD,
          "selectedunplayedsong": curses.color_pair(3) | curses.A_BOLD,
          "playedsong":           curses.A_NORMAL,
          "selectedplayedsong":   curses.color_pair(3),
          },
    "iteminfo": {
          "background":           curses.color_pair(1),
          "border":               curses.color_pair(5),
          "activeborder":         curses.color_pair(5) | curses.A_BOLD,
          "title":                curses.color_pair(5) | curses.A_BOLD,
          "description":          curses.color_pair(6) | curses.A_BOLD,
          "content":              curses.A_NORMAL,
          },
    "player": {
          "background":           curses.color_pair(1),
          "border":               curses.color_pair(5),
          "activeborder":         curses.color_pair(5) | curses.A_BOLD,
          "title":                curses.color_pair(5) | curses.A_BOLD,
          "description":          curses.color_pair(6) | curses.A_BOLD,
          "content":              curses.A_NORMAL,
          "progressbar":          curses.color_pair(9),
          "progressbarhigh":      curses.color_pair(10),
          },
    "help": {
          "background":           curses.color_pair(1),
          "border":               curses.color_pair(5),
          "activeborder":         curses.color_pair(5) | curses.A_BOLD,
          "title":                curses.color_pair(5) | curses.A_BOLD,
          "key":                  curses.color_pair(6) | curses.A_BOLD,
          "description":          curses.A_NORMAL,
          },
    "mixer": {
          "background":           curses.color_pair(1),
          "border":               curses.color_pair(5),
          "activeborder":         curses.color_pair(5) | curses.A_BOLD,
          "title":                curses.color_pair(5) | curses.A_BOLD,
          "description":          curses.color_pair(6) | curses.A_BOLD,
          "content":              curses.A_NORMAL,
          "bar":                  curses.color_pair(9),
          "barhigh":              curses.color_pair(10),
          },
    "input": {
          "background":           curses.color_pair(1),
          "border":               curses.color_pair(5),
          "activeborder":         curses.color_pair(5) | curses.A_BOLD,
          "title":                curses.color_pair(5) | curses.A_BOLD,
          "description":          curses.color_pair(6) | curses.A_BOLD,
          "content":              curses.A_NORMAL,
          },
    "statusbar": {
          "background":           curses.color_pair(1),
          "key":                  curses.color_pair(6) | curses.A_BOLD,
          "description":          curses.A_NORMAL,
          },
    }

# example attribute/color scheme for monochrome terminals
colors_mono = {
    "filelist": {
          "background":           curses.A_NORMAL,
          "border":               curses.A_NORMAL,
          "activeborder":         curses.A_BOLD,
          "title":                curses.A_BOLD,
          "scrollbar":            curses.A_NORMAL,
          "scrollbarhigh":        curses.A_BOLD,
          "scrollbararrow":       curses.A_BOLD,
          "song":                 curses.A_NORMAL,
          "directory":            curses.A_BOLD,
          "selectedsong":         curses.A_REVERSE,
          "selecteddirectory":    curses.A_REVERSE | curses.A_BOLD,
          },
    "playlist": {
          "background":           curses.A_NORMAL,
          "border":               curses.A_NORMAL,
          "activeborder":         curses.A_BOLD,
          "title":                curses.A_BOLD,
          "scrollbar":            curses.A_NORMAL,
          "scrollbarhigh":        curses.A_BOLD,
          "scrollbararrow":       curses.A_BOLD,
          "unplayedsong":         curses.A_BOLD,
          "selectedunplayedsong": curses.A_BOLD | curses.A_REVERSE,
          "playedsong":           curses.A_NORMAL,
          "selectedplayedsong":   curses.A_REVERSE,
          },
    "iteminfo": {
          "background":           curses.A_NORMAL,
          "border":               curses.A_NORMAL,
          "activeborder":         curses.A_BOLD,
          "title":                curses.A_BOLD,
          "description":          curses.A_BOLD,
          "content":              curses.A_NORMAL,
          },
    "player": {
          "background":           curses.A_NORMAL,
          "border":               curses.A_NORMAL,
          "activeborder":         curses.A_BOLD,
          "title":                curses.A_BOLD,
          "description":          curses.A_BOLD,
          "content":              curses.A_NORMAL,
          "progressbar":          curses.A_NORMAL,
          "progressbarhigh":      curses.A_BOLD,
          },
    "help": {
          "background":           curses.A_NORMAL,
          "border":               curses.A_NORMAL,
          "activeborder":         curses.A_BOLD,
          "title":                curses.A_BOLD,
          "key":                  curses.A_BOLD,
          "description":          curses.A_NORMAL,
          },
    "mixer": {
          "background":           curses.A_NORMAL,
          "border":               curses.A_NORMAL,
          "activeborder":         curses.A_BOLD,
          "title":                curses.A_BOLD,
          "description":          curses.A_BOLD,
          "content":              curses.A_NORMAL,
          "bar":                  curses.A_NORMAL,
          "barhigh":              curses.A_BOLD,
          },
    "input": {
          "background":           curses.A_NORMAL,
          "border":               curses.A_NORMAL,
          "activeborder":         curses.A_BOLD,
          "title":                curses.A_BOLD,
          "description":          curses.A_BOLD,
          "content":              curses.A_NORMAL,
          },
    "statusbar": {
          "background":           curses.A_NORMAL,
          "key":                  curses.A_BOLD,
          "description":          curses.A_NORMAL,
          
          }
    }

# automagically select one the two attribute/color schemes defined above
# Of course, you can also select one scheme manually, for instance if you
# are still a fan of the old monochrome look of PyTone <=1.9.6 ...
if curses.has_colors:
    colors = colors_colors
else:
    colors = colors_mono

