#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses

import events
import filelist
import statusbar
import window

from helper import debug

class filelistwin(window.window):
    def __init__(self,
                 screen, h, w, y, x,
                 colors,
                 hasscrollbar,
                 channel,
                 mp3basedir, songdb, keybindings):
        self.channel = channel
        self.mp3base = mp3basedir
        self.songdb = songdb
        self.keybindings = keybindings
        self.searchstring = None
        self.lastadded = None

        window.window.__init__(self,
                               screen, h, w, y, x, colors,
                               "MP3s", hasscrollbar)
        
        self.items = filelist.dbbasedfilelist(self, mp3basedir, songdb)

        self.channel.subscribe(events.keypressed, self.keypressed)
        self.channel.subscribe(events.mouseevent, self.mouseevent)
        self.channel.subscribe(events.focuschanged, self.focuschanged)

    def updatestatusbar(self):
        sbar = []
        if len(self.items.shistory)>0:
            sbar += statusbar.generatedescription("filelist", "dirup")
            sbar += statusbar.separator
        if self.items.isdirselected():
            sbar += statusbar.generatedescription("filelist", "dirdown")
            sbar += statusbar.separator
            sbar += statusbar.generatedescription("filelist", "adddirtoplaylist")
            sbar += statusbar.separator
        elif self.items.issongselected():
            sbar += statusbar.generatedescription("filelist", "addsongtoplaylist")
            sbar += statusbar.separator

        sbar += statusbar.generatedescription("filelist", "activateplaylist")
        sbar += statusbar.separator
        sbar += statusbar.generatedescription("global", "playerstart")
        sbar += statusbar.separator
        sbar += statusbar.generatedescription("global", "showhelp")

        events.hub.notify(events.updatestatusbar(sbar))

    def updatescrollbar(self):
        self.drawscrollbar(self.items.top, len(self.items))

    def searchhandler(self, searchstring, key):
        if key==ord("\n"):
            events.hub.notify(events.activatefilelist())
            self.items.selectbysearchstring(searchstring)
            self.update()

    def isclickonstring(self, y, x):
        """ check whether a click was on a string or not """
        while x<self.ix+self.iw-self.hasscrollbar:
            if self.win.inch(y, x) & 0xFF!=32:
                return 1
            x += 1
        return 0

    # event handler

    def keypressed(self, event):
        if self.hasfocus():
            key = event.key
            if key in self.keybindings["selectnext"]:
                self.items.selectnext()
            elif key in self.keybindings["selectprev"]:
                self.items.selectprev()
            elif key in self.keybindings["selectnextpage"]:
                self.items.selectnextpage()
            elif key in self.keybindings["selectprevpage"]:
                self.items.selectprevpage()
            elif key in self.keybindings["selectfirst"]:
                self.items.selectfirst()
            elif key in self.keybindings["selectlast"]:
                self.items.selectlast()
            elif key in self.keybindings["dirdown"] and \
                     self.items.isdirselected():
                self.items.dirdown()
            elif key in self.keybindings["dirup"]:
                self.items.dirup()
            elif key in self.keybindings["addsongtoplaylist"] and \
                     self.items.issongselected():
                songtoadd = self.items.getselected()
                if songtoadd is not self.lastadded:
                    self.lastadded = songtoadd
                    events.hub.notify(events.playlistaddsong(songtoadd))
                    self.items.selectrelative(+1)
            elif key in self.keybindings["adddirtoplaylist"] and \
                     self.items.isdirselected():
                itemtoadd = self.items.getselected()
                if itemtoadd is not self.lastadded:
                    self.lastadded = itemtoadd
                    self.items.insertrecursiveselection()
                    self.items.selectrelative(+1)
            elif key in self.keybindings["activateplaylist"]:
                events.hub.notify(events.activateplaylist())
            elif key in self.keybindings["toggleview"]:
                if isinstance(self.items, filelist.dirbasedfilelist):
                    del self.items
                    self.items = filelist.dbbasedfilelist(self, self.mp3base, self.songdb)
                else:
                    del self.items
                    self.items = filelist.dirbasedfilelist(self, self.mp3base, self.songdb)
            elif key in self.keybindings["generaterandomlist"]:
                self.items.generaterandomlist()
            elif key in self.keybindings["search"]:
                events.hub.notify(events.requestinput(_("Search"),
                                                      "",
                                                      self.searchhandler))
            elif ord("a")<=key-1024<=ord("z") or ord("A")<=key-1024<=ord("Z") :
                self.items.selectbyletter(chr(key-1024))
            else:
                return
        
            self.update()
            raise events.TerminateEventProcessing

    def mouseevent(self, event):
        if self.enclose(event.y, event.x):
            y, x = self.stdscrtowin(event.y, event.x)
            self.top()

            if event.state & curses.BUTTON1_CLICKED:
                if x==self.ix+self.iw and self.hasscrollbar:
                    scrollbarbegin = 2 + self.ih*self.items.top/len(self.items)
                    scrollbarend = scrollbarbegin + self.ih*self.ih/len(self.items)-2

                    if y==self.iy:
                        self.items.selectprev()
                    elif y==self.iy+self.ih-1:
                        self.items.selectnext()                    
                    elif self.iy<y<scrollbarbegin:
                        self.items.selectprevpage()
                    elif scrollbarend<y<self.iy+self.ih-1:
                        self.items.selectnextpage()
                elif self.items.selectbylinenumber(y-self.iy) and \
                   self.isclickonstring(y, x) and \
                   self.items.isdirselected():
                    self.items.dirdown()
            elif event.state & curses.BUTTON1_DOUBLE_CLICKED:
                if self.items.selectbylinenumber(y-self.iy) and \
                   self.isclickonstring(y, x):
                    if self.items.issongselected():
                        songtoadd = self.items.getselected()
                        self.lastadded = None
                        events.hub.notify(events.playlistaddsong(songtoadd))
                    else:
                        self.lastadded = None
                        self.items.insertrecursiveselection()
            elif event.state & curses.BUTTON3_CLICKED:
                self.items.dirup()
            else:
                return
            
            self.update()
            raise events.TerminateEventProcessing

    def focuschanged(self, event):
        if self.hasfocus():
            self.lastadded = None
            events.hub.notify(events.selectionchanged(self.items.getselected()))
        self.update()
        
    # window update method

    def update(self):
        self.settitle(self.items.selectionpath())
        window.window.update(self)

        if self.hasfocus():
            self.updatestatusbar()
        
        for i in range(self.items.top, self.items.top+self.ih):
            attr = curses.A_NORMAL
            if i<len(self.items):
                item = self.items[i]
                if item.isdir():
                    name = item.name + "/"
                    if i==self.items.selected and self.hasfocus():
                        attr = self.colors["selecteddirectory"]
                    else:
                        attr = self.colors["directory"]
                else:
                    name = item.title
                    if i==self.items.selected and self.hasfocus():
                        attr = self.colors["selectedsong"]
                    else:
                        attr = self.colors["song"]
            else:
                name = ""
            self.win.addstr(i-self.items.top+1, 1, name.ljust(self.iw)[:self.iw], attr)

        self.updatescrollbar()
