#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os.path
import string
import re
import time

import mp3info

import events

from helper import formattime, debug

tracknrandtitlere = re.compile("^\[?(\d+)\]?-(.*)\.mp3$")
UNKNOWN = "Unknown"

def capword(word):
    for i in range(len(word)):
        if word[i] not in string.punctuation: break
    else:
        i = len(word)
    return word[:i+1].upper() + word[i+1:]

def capitalize(s):
    """capitalize first character of word, leave rest intact"""
    return string.join([capword(word) for word in s.split()])

class song:
    def __init__(self, path):
        assert os.path.isfile(path), "'%s' doesn't exist" % path
        self.path = os.path.normpath(path)
        self.name = os.path.basename(path)
        
        match = re.match(tracknrandtitlere, self.name)
        if match:
            fntracknr = str(int(match.group(1)))
            fntitle = match.group(2)
        else:
            fntracknr = ""
            fntitle = self.name
	    if fntitle.endswith(".mp3"):
	        fntitle = fntitle[:-4]

        try:
            id3info = mp3info.get_mp3tag(self.path)
            self.title = capitalize(id3info.get('TITLE', fntitle))
            self.album = capitalize(id3info.get('ALBUM', UNKNOWN))
            self.artist = capitalize(id3info.get('ARTIST', UNKNOWN))
            self.year = id3info.get('YEAR', "")
            self.genre  = id3info.get('GENRE', "")
            self.tracknr = id3info.get('TRACKNUMBER', fntracknr)
        except:
            self.title = fntitle
            self.album = UNKNOWN
            self.artist = UNKNOWN
            self.year = ""
            self.genre = ""
            self.tracknr = fntracknr

        # do some further treatment of the song info

        # use title from filename, if this is only a longer version of
        # the id3 tag title
        if fntitle.startswith(self.title):
	    self.title = fntitle

        # strip leading "The " in artist names, often used inconsistently
        if self.artist.startswith("The "):
            self.artist = self.artist[4:]

        # we don't want empty album names
        if self.album=="":
            self.album = UNKNOWN

        # nor empty artist names
        if self.artist=="":
            self.artist = UNKNOWN

        # get the length of the MP3 file
        try:
            mp3i = mp3info.mp3info(self.path)
            self.length = (mp3i["MM"]*60+mp3i["SS"])*1000
        except:
            self.length = 0

        self.nrplayed = 0
        self.lastplayed = None

    def __repr__(self):
        return self.path

    def play(self):
        self.nrplayed += 1
        self.lastplayed = time.time()
        events.hub.notify(events.songupdate(self))

class artist:
    def __init__(self, name, path=None):
        self.name = name
        self.albums = {}
        self.path = path

class album:
    def __init__(self, artist, name, path=None):
        self.name = name
        self.artist = artist
        self.songs = []
        self.path = path


