#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses

import window
import events
from helper import formattime


class playerwin(window.window):
    def __init__(self, screen, h, w, y, x, colors, channel, player, keybindings):
        self.playing = 0
        self.title = None
        self.ttime = None
        self.ptime = None
        self.player = player
        self.keybindings = keybindings["global"]
        window.window.__init__(self, screen, h, w, y, x,
                               colors,
                               _("Playback Info"))

        # we don't want to have the focus
        self.panel.bottom()
        
        channel.subscribe(events.playbackinfochanged, self.playbackinfochanged)
        channel.subscribe(events.keypressed, self.keypressed)
        self.update()

    def playbackinfochanged(self, event):
        if event.player is self.player:
            self.playing = event.playing
            if self.playing:
                self.title = event.title or _("Playback Info")
            else:
                self.title = _("Playback Info")
            self.ptime = event.ptime
            self.ttime = event.ttime
            self.update()

    def keypressed(self, event):
        key = event.key
        if key in self.keybindings["playerstart"]:
            events.hub.notify(events.playerstart(self.player))
        elif key in self.keybindings["playernextsong"]:
            events.hub.notify(events.playerforward(self.player))   
        elif key in self.keybindings["playerstop"]:
            events.hub.notify(events.playerstop(self.player))   
        else:
            return
        raise events.TerminateEventProcessing
        
    def update(self):
        window.window.update(self)
        self.win.addstr(1, 1, " "*self.iw)
        if self.playing:
            self.win.move(1, 1)
            s1 = _("Time:")
            s2 = " %s/%s " % (formattime(self.ptime), formattime(self.ttime))
            self.win.addstr(s1, self.colors["description"])
            self.win.addstr(s2, self.colors["content"])
            
            barlen = self.iw-len(s1)-len(s2)
            percentplayed = barlen*self.ptime/self.ttime
            self.win.addstr("#"*(percentplayed+1), self.colors["progressbarhigh"])
            self.win.addstr("#"*(barlen-percentplayed-1), self.colors["progressbar"])
