#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses

import events
import playlist
import statusbar
import window

from helper import formattime, debug

class playlistwin(window.window):
    def __init__(self,
                 screen, h, w, y, x,
                 colors,
                 hasscrollbar,
                 channel,
                 songdb, logfilename, keybindings):
        self.channel = channel
        self.songdb = songdb
        
        # we both need the playlist and the global keybindings
        self.keybindings = keybindings
        
        window.window.__init__(self,
                               screen, h, w, y, x,
                               colors,
                               _("Playlist"), hasscrollbar)
        
        self.playlist = playlist.playlist(self, songdb, logfilename)
        
        self.channel.subscribe(events.keypressed, self.keypressed)
        self.channel.subscribe(events.mouseevent, self.mouseevent)
        self.channel.subscribe(events.focuschanged, self.focuschanged)

    def updatestatusbar(self):
        sbar = []
        if self.playlist.selected is not None:
            sbar += statusbar.generatedescription("playlist", "deleteitem")
            sbar += statusbar.separator
            sbar += statusbar.generatedescription("playlist", "moveitemup")
            sbar += statusbar.separator
            sbar += statusbar.generatedescription("playlist", "moveitemdown")
            sbar += statusbar.separator

        sbar += statusbar.generatedescription("playlist", "activatefilelist")
        sbar += statusbar.separator
        sbar += statusbar.generatedescription("global", "playerstart")
        sbar += statusbar.separator
        sbar += statusbar.generatedescription("global", "showhelp")

        events.hub.notify(events.updatestatusbar(sbar))

    def updatescrollbar(self):
        self.drawscrollbar(self.playlist.top, len(self.playlist))

    # event handlers

    def keypressed(self, event):
        if self.hasfocus():
            key = event.key

            if key in self.keybindings["selectnext"]:
                self.playlist.selectnext()
            elif key in self.keybindings["selectprev"]:
                self.playlist.selectprev()
            elif key in self.keybindings["selectnextpage"]:
                self.playlist.selectnextpage()
            elif key in self.keybindings["selectprevpage"]:
                self.playlist.selectprevpage()
            elif key in self.keybindings["selectfirst"]:
                self.playlist.selectfirst()
            elif key in self.keybindings["selectlast"]:
                self.playlist.selectlast()
            elif key in self.keybindings["activatefilelist"]:
                events.hub.notify(events.activatefilelist())
            elif key in self.keybindings["moveitemup"]:
                self.playlist.moveitemup()
            elif key in self.keybindings["moveitemdown"]:
                self.playlist.moveitemdown()
            elif key in self.keybindings["deleteitem"]:
                self.playlist.deleteselected()
            else:
                return
            
            self.update()
            raise events.TerminateEventProcessing

    def mouseevent(self, event):
        if self.enclose(event.y, event.x):
            y, x = self.stdscrtowin(event.y, event.x)
            self.top()

            if event.state & curses.BUTTON1_CLICKED:
                if self.playlist.selectbylinenumber(y-1):
                    pass

            elif event.state & curses.BUTTON1_DOUBLE_CLICKED:
                if self.playlist.selectbylinenumber(y-1):
                    pass
            elif event.state & curses.BUTTON3_CLICKED:
                pass
            else:
                return
            
            self.update()
            raise events.TerminateEventProcessing

    def focuschanged(self, event):
        if self.hasfocus():
            events.hub.notify(events.selectionchanged(self.playlist.getselected()))
        self.update()

    # window update method

    def update(self):
        self.settitle("%s (-%s/%s)" % ( _("Playlist"),
                                        formattime((self.playlist.ttime-
                                                    self.playlist.ptime)/1000),
                                        formattime(self.playlist.ttime/1000)))

        window.window.update(self)
        if self.hasfocus():
            self.updatestatusbar()
            
        for i in range(self.playlist.top, self.playlist.top+self.ih):
            attr = curses.A_NORMAL

            if i<len(self.playlist):
                song = self.playlist[i]
                name = song.title
                if song.hasbeenplayed():
                    if i==self.playlist.selected and self.hasfocus():
                        attr = self.colors["selectedplayedsong"]
                    else:
                        attr = self.colors["playedsong"]
                else:
                    if i==self.playlist.selected and self.hasfocus():
                        attr = self.colors["selectedunplayedsong"]
                    else:
                        attr = self.colors["unplayedsong"]
            else:
                name = ""
            self.win.addstr(i-self.playlist.top+1, 1,
                            name.ljust(self.iw)[:self.iw],
                            attr)

        self.updatescrollbar()
